@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2016 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2017 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -179,6 +179,7 @@ extern char** environ;
 #define WRAPPER_ACTION_SOURCE_CODE_COMMANDFILE             2  /* Action originated from a commandfile. */
 #define WRAPPER_ACTION_SOURCE_CODE_WINDOWS_SERVICE_MANAGER 3  /* Action originated from the Windows Service Manager. */
 #define WRAPPER_ACTION_SOURCE_CODE_ON_EXIT                 4  /* Action originated from an on_exit configuration. */
+#define WRAPPER_ACTION_SOURCE_CODE_SIGNAL                  5  /* Action originated from a signal. */
 #define WRAPPER_ACTION_SOURCE_CODE_PING_TIMEOUT            11 /* Action originated from a timeout. */
 
 /* Because of the way time is converted to ticks, the largest possible timeout that
@@ -263,8 +264,6 @@ struct WrapperConfig {
     int     commandLogLevel;        /* The log level to use when logging the java command. */
     int     printJVMVersion;        /* tells the Wrapper to create a temp JVM to query the version, before starting the java application */
 #ifdef WIN32
-    int     argc;                   /* argc params as originally received in the entry function */
-    TCHAR   **argv;                 /* argv params as originally received in the entry function */
     TCHAR   *jvmVersionCommand;     /* Command used to launch the JVM and request its version */
     TCHAR   *jvmCommand;            /* Command used to launch the JVM */
 #else /* UNIX */
@@ -291,6 +290,7 @@ struct WrapperConfig {
     int     jvmExitTimeout;         /* Number of seconds the wrapper will wait for a JVM to process to terminate */
     int     jvmCleanupTimeout;      /* Number of seconds the wrapper will allow for its post JVM shudown cleanup. */
     int     jvmTerminateTimeout;    /* Number of seconds the wrapper will allow for the JVM to respond to TerminateProcess request. */
+    int     jvmSilentKill;          /* TRUE if the JVM should be silently killed at the next opportunity. */
 #ifdef WIN32
     int     javaIOBufferSize;       /* Size of the pipe buffer to use for java I/O. */
 #endif
@@ -348,8 +348,13 @@ struct WrapperConfig {
     int     startupDelayConsole;    /* Delay in seconds before starting the first JVM in console mode. */
     int     startupDelayService;    /* Delay in seconds before starting the first JVM in service mode. */
     int     exitCode;               /* Code which the wrapper will exit with */
+    int     errorExitCode;          /* Code which the wrapper will exit with when there is an error. */
     int     exitRequested;          /* TRUE if the current JVM should be shutdown. */
     int     restartRequested;       /* WRAPPER_RESTART_REQUESTED_NO, WRAPPER_RESTART_REQUESTED_AUTOMATIC, or WRAPPER_RESTART_REQUESTED_CONFIGURED if the another JVM should be launched after the current JVM is shutdown. Only set if exitRequested is set. */
+    int     shutdownActionTriggered; /* TRUE if any action causing a JVM exit was triggered by the Wrapper (see the WRAPPER_ACTION_SOURCE_CODE_* definitions above).
+                                     *  This flag indicates that the action specified by wrapper.on_exit should be ignored. It must be reset after the JVM is
+                                     *  restarted so that wrapper.on_exit can continue to catch JVM exit codes. */
+    TCHAR*  shutdownActionPropertyName; /* Keep the name of the property that originated the shudown for logging. */
     int     stoppedPacketReceived;  /* TRUE if the STOPPED packet was received before a restart. */
     int     restartPacketReceived;  /* TRUE if the RESTART packet was received before a restart. */
     int     jvmRestarts;            /* Number of times that a JVM has been launched since the wrapper was started. */
@@ -427,7 +432,16 @@ struct WrapperConfig {
     int     jvmConsoleVisible;      /* True if the JVM Console window is visible. */
     int     ntAllocConsole;         /* True if a console should be allocated for the Service. */
     int     generateConsole;        /* Make sure that a console is always generated to support thread dumps */
+#ifndef WINIA
+    int     ntPreshutdown;          /* True if the service accepts SERVICE_CONTROL_PRESHUTDOWN controle code, False if it accepts SERVICE_CONTROL_SHUTDOWN. */
+    int     ntPreshutdownTimeout;   /* Number of seconds that the service controler will wait after sending the SERVICE_CONTROL_PRESHUTDOWN notification. */
+#endif
+    int     ntShutdownWaitHint;     /* Delay in seconds requested by the Wrapper when it reports a STOPPING or PAUSING status to the service controler. */
+    int     ntStartupWaitHint;      /* Delay in seconds requested by the Wrapper when it reports a STARTING or RESUMING status to the service controler. */
     int     threadDumpControlCode;  /* Control code which can be used to trigger a thread dump. */
+    BOOL    DEPApiAvailable;        /* TRUE if the DEP API is available in Kernel32, false otherwise. */
+    BOOL    DEPStatus;              /* TRUE if DEP has been enabled. */
+    DWORD   DEPError;               /* Error code returned when failing to enable DEP. */
 #else /* UNIX */
     int     daemonize;              /* TRUE if the process  should be spawned as a daemon process on launch. */
     int     signalHUPMode;          /* Controls what happens when the Wrapper receives a HUP signal. */
@@ -456,10 +470,14 @@ struct WrapperConfig {
     int     ctrlCodeContinueTrapped;/* SERVICE_CONTROL_CONTINUE was trapped. */
     int     ctrlCodeStopTrapped;    /* SERVICE_CONTROL_STOP was trapped. */
     int     ctrlCodeShutdownTrapped;/* SERVICE_CONTROL_SHUTDOWN was trapped. */
+ #ifndef WINIA
+    int     ctrlCodePreShutdownTrapped;/* SERVICE_CONTROL_PRESHUTDOWN was trapped. */
+ #endif
     int     ctrlCodeDumpTrapped;    /* The configured thread dump control code was trapped. */
 #else
     int     signalInterruptTrapped; /* SIGINT was trapped. */
     int     signalQuitTrapped;      /* SIGQUIT was trapped. */
+    int     signalQuitKernel;       /* SIGQUIT was originated by the kernel. */
     int     signalChildTrapped;     /* SIGCHLD was trapped. */
     int     signalTermTrapped;      /* SIGTERM was trapped. */
     int     signalHUPTrapped;       /* SIGHUP was trapped. */
@@ -473,6 +491,9 @@ struct WrapperConfig {
 #define WRAPPER_SIGNAL_MODE_RESTART  (char)101
 #define WRAPPER_SIGNAL_MODE_SHUTDOWN (char)102
 #define WRAPPER_SIGNAL_MODE_FORWARD  (char)103
+#define WRAPPER_SIGNAL_MODE_PAUSE    (char)104
+#define WRAPPER_SIGNAL_MODE_RESUME   (char)105
+#define WRAPPER_SIGNAL_MODE_CLOSE_LOGFILE (char)106
 
 #define WRAPPER_MSG_START         (char)100
 #define WRAPPER_MSG_STOP          (char)101
@@ -601,29 +622,6 @@ extern int wrapperWildcardMatch(const TCHAR *text, const TCHAR *pattern, size_t
  */
 extern size_t wrapperGetMinimumTextLengthForPattern(const TCHAR *pattern);
 
-/**
- * Function that copies an array of strings.
- *  The output will be pointer to a NULL terminated array even though the source does not end with a NULL value.
- *
- * @param arrOut Array with copied strings
- * @param arrIn  Array to copy
- * @param count  Number of elements to copy. If arrIn has less elements than count, arrOut will be the same size as arrIn.
- */
-extern void wrapperCopyStringArray(TCHAR*** arrOut, TCHAR*** arrIn, int count);
-
-/**
- * Function that splits a text into tokens according to delimiters.
- *  Any spaces around delimiters will be trimmed.
- *  Several delimiters following each others without spaces will be considered as one.
- *  If there are any space between 2 delimiters, the function will consider it as a token and trim it.
- *
- * @param text Text to be split
- * @param delim Array of delimiters 
- *
- * @return tokens in an Array of TCHAR*
- */
-/*extern TCHAR** wrapperSplitText(const TCHAR* text, const TCHAR* delim);*/
-
 /**
  * Trims any whitespace from the beginning and end of the in string
  *  and places the results in the out buffer.  Assumes that the out
@@ -656,6 +654,7 @@ extern int initializeStartup();
 extern void disposeStartup();
 #endif
 extern void disposeJavaIO();
+extern int initializeTimer();
 extern void disposeTimer();
 
 extern int showHostIds(int logLevel);
@@ -679,11 +678,12 @@ extern int *wrapperGetActionListForNames(const TCHAR *actionNameList, const TCHA
  *                   custom events.
  * @param triggerMsg The reason the actions are being fired.
  * @param actionSourceCode Tracks where the action originated.
+ * @param actionPropertyIndex Index of the property where the action was configured. Ignored if the type of action is not configured with a <n>-component property.
  * @param logForActionNone Flag stating whether or not a message should be logged
  *                         for the NONE action.
  * @param exitCode Error code to use in case the action results in a shutdown.
  */
-extern void wrapperProcessActionList(int *actionList, const TCHAR *triggerMsg, int actionSourceCode, int logForActionNone, int exitCode);
+extern void wrapperProcessActionList(int *actionList, const TCHAR *triggerMsg, int actionSourceCode, int actionPropertyIndex, int logForActionNone, int exitCode);
 
 extern void wrapperAddDefaultProperties();
 
@@ -691,13 +691,12 @@ extern int wrapperLoadConfigurationProperties(int preload);
 
 extern void wrapperGetCurrentTime(struct timeb *timeBuffer);
 
+extern void updateStringValue(TCHAR **ptr, const TCHAR *value);
+
 #ifdef WIN32
 
 extern void wrapperInitializeProfileCounters();
 extern void wrapperDumpPageFaultUsage();
-
-extern void updateStringValue(TCHAR **ptr, const TCHAR *value);
-
 extern TCHAR** wrapperGetSystemPath();
 extern int wrapperGetJavaHomeFromWindowsRegistry(TCHAR *javaHome);
 #endif
@@ -798,14 +797,6 @@ extern void wrapperSetJavaState(int jState, TICKS nowTicks, int delay);
  * Platform specific methods
  *****************************************************************************/
 #ifdef WIN32
-/**
- * Allocates an hidden console. (fix for the console flicker bug).
- *  The size of the console will be minimized and its position will be set outside of the screen.
- *  
- * @return TRUE if the console was allocated, FALSE if it could not be allocated.
- */
-extern int wrapperAllocHiddenConsole();
-
 extern void wrapperCheckConsoleWindows();
 /**
  *   checks the digital Signature of the binary and reports the result.
@@ -926,10 +917,14 @@ extern int wrapperKillProcessNow();
 /**
  * Puts the Wrapper into a state where the JVM will be killed at the soonest
  *  possible opportunity.  It is necessary to wait a moment if a final thread
- *  dump is to be requested.  This call wll always set the JVM state to
+ *  dump is to be requested.  This call will always set the JVM state to
  *  WRAPPER_JSTATE_KILLING.
+ *
+ * @param silent TRUE to skip messages saying that the JVM did not exit on request.
+ *               This is useful in certain cases where we kill the JVM without trying
+ *               to shut it down cleanly.
  */
-extern void wrapperKillProcess();
+extern void wrapperKillProcess(int silent);
 
 /**
  * Launch the wrapper as a console application.
@@ -978,6 +973,20 @@ extern void wrapperStopProcess(int exitCode, int force);
  */
 extern const TCHAR *wrapperGetRestartProcessMessage();
 
+/**
+ * Depending on the current state, we want to change the exact message displayed when pausing the Wrapper.
+ *
+ * The logic here needs to match that in wrapperPauseProcess.
+ */
+extern const TCHAR *wrapperGetPauseProcessMessage();
+
+/**
+ * Depending on the current state, we want to change the exact message displayed when resuming the Wrapper.
+ *
+ * The logic here needs to match that in wrapperResumeProcess.
+ */
+extern const TCHAR *wrapperGetResumeProcessMessage();
+
 /**
  * Used to ask the state engine to shut down the JVM.
  */
@@ -1122,7 +1131,6 @@ extern void wrapperStoppedSignaled();
 extern void wrapperStartPendingSignaled(int waitHint);
 extern void wrapperStartedSignaled();
 
-
 /******************************************************************************
  * Inner types and methods for loading Wrapper configuration.
  *****************************************************************************/
